@@ -194,7 +194,7 @@ html_view_relayout (HtmlView *view)
 		
 		HTML_BOX_ROOT (view->root)->min_width = GTK_WIDGET (view)->allocation.width;
 		HTML_BOX_ROOT (view->root)->min_height = GTK_WIDGET (view)->allocation.height;
-		
+
 		relayout = html_relayout_new ();
 		relayout->type = HTML_RELAYOUT_INCREMENTAL;
 		
@@ -226,6 +226,13 @@ html_view_relayout (HtmlView *view)
 		gtk_idle_remove (view->relayout_idle_id);
 		view->relayout_idle_id = 0;
 	}
+
+	if (GTK_WIDGET_HAS_FOCUS (view)) {
+		if (view->document->focus_element == NULL) {
+			html_document_update_focus_element (view->document, html_focus_iterator_next_element (view->document->dom_document, NULL));
+			html_view_focus_element (view);
+		}
+	}
 }
 
 static gint
@@ -640,6 +647,15 @@ html_view_realize (GtkWidget *widget)
 	HtmlView *view;
 
 	view = HTML_VIEW (widget);
+
+	/*
+	 * GtkLayout uses the bg color for background but we want
+	 * to use base color.
+	 */	
+ 	widget->style = gtk_style_copy (widget->style);
+	
+ 	widget->style->bg[GTK_STATE_NORMAL] =
+ 		widget->style->base[GTK_STATE_NORMAL];
 	
 	if (GTK_WIDGET_CLASS (parent_class)->realize)
 		(* GTK_WIDGET_CLASS (parent_class)->realize) (widget);
@@ -828,34 +844,36 @@ html_view_focus (GtkWidget *widget, GtkDirectionType direction)
 {
 	HtmlView *view = HTML_VIEW (widget);
 	HtmlBox *box;
-	DomElement *old_focus_element;
+	DomElement *new_focus_element = NULL;
 
-	if (!GTK_WIDGET_HAS_FOCUS (widget)) {
-		gtk_widget_grab_focus (widget);
-		return TRUE;
-	} else if (view->document == NULL || view->document->dom_document == NULL) {
+        if (view->document == NULL || view->document->dom_document == NULL) {
 		return FALSE;
 	}
 
-	old_focus_element = view->document->focus_element;
 	if (direction == GTK_DIR_TAB_FORWARD) {
-		html_document_update_focus_element (view->document, html_focus_iterator_next_element (view->document->dom_document, view->document->focus_element));
-		html_view_focus_element (view);
-		return TRUE;
+		new_focus_element = html_focus_iterator_next_element (view->document->dom_document, view->document->focus_element);
+		/* If end is reached cycle around */
+		if (new_focus_element == NULL)
+			new_focus_element = html_focus_iterator_next_element (view->document->dom_document, NULL);
 	}
 	else if (direction == GTK_DIR_TAB_BACKWARD) {
-		html_document_update_focus_element (view->document, html_focus_iterator_prev_element (view->document->dom_document, view->document->focus_element));
-
-		html_view_focus_element (view);
-		return TRUE;
+		new_focus_element = html_focus_iterator_prev_element (view->document->dom_document, view->document->focus_element);
+		/* If start is reached cycle around */
+		if (new_focus_element == NULL)
+			new_focus_element = html_focus_iterator_prev_element (view->document->dom_document, NULL);
 	}
 
-	/* Handle special case when we haven't got any focusable elements */
-	if (old_focus_element == NULL) {
+	/* Handle special case when we haven't any focusable elements */
+	if (new_focus_element == NULL) {
 		if (!GTK_WIDGET_HAS_FOCUS (widget)) {
 			gtk_widget_grab_focus (widget);
 			return TRUE;
-		}
+		} 
+	} else { 
+		html_document_update_focus_element (view->document, new_focus_element);
+
+		html_view_focus_element (view);
+		return TRUE;
 	}
 
 	return FALSE;
@@ -1312,6 +1330,7 @@ html_view_focus_element (HtmlView *view)
 		GtkAdjustment *adj = GTK_LAYOUT (view)->vadjustment;
 
 		set_adjustment_clamped (adj, 0);
+		gtk_widget_grab_focus (GTK_WIDGET (view));
 	}
 	gtk_widget_queue_draw (GTK_WIDGET (view));
 }
